using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;

namespace PhotoViewer
{
	public class PhotoGridDelegate : IGGridViewDelegate
	{
		PhotoViewerViewController _vc;

		public PhotoGridDelegate(PhotoViewerViewController vc) :base()
		{
			_vc = vc;
		}

		public override void PageChanged (IGGridView gridView, IGCellPath path)
		{
			_vc.PageChanged (path);
		}
	}

	public class ThumbGridDelegate : IGGridViewDelegate
	{
		PhotoViewerViewController _vc;

		public ThumbGridDelegate(PhotoViewerViewController vc) :base()
		{
			_vc = vc;
		}

		public override void InitializeCell (IGGridView gridView, IGGridViewCell cell)
		{
			cell.SelectedColor = UIColor.FromWhiteAlpha (1, .9f);
		}

		public override void DidSelectCell (IGGridView gridView, IGCellPath path)
		{
			_vc.ThumbSelected (path);
		}

	}

	public class PhotoInfo : NSObject
	{
		[Export("imagePath")]
		public string ImagePath 
		{
			get;
			set;
		}

		[Export("thumbPath")]
		public string ThumbPath 
		{
			get;
			set;
		}

		[Export("image")]
		public UIImage Image 
		{
			get 
			{
				UIImage img = UIImage.FromBundle (this.ImagePath);
				return img;
			}

		}

		[Export("thumb")]
		public UIImage Thumb 
		{
			get 
			{
				return UIImage.FromBundle (this.ThumbPath);
			}
		}
	}

	public partial class PhotoViewerViewController : UIViewController
	{
		IGGridView _photoGridView;
		IGGridViewSingleRowSingleFieldDataSourceHelper _photoDS;

		IGGridView _thumbsGridView;
		IGGridViewSingleRowSingleFieldDataSourceHelper _singleRowThumbDS;
		IGGridViewSingleFieldMultiColumnDataSourceHelper _thumbsDS;

		int _numberOfColumns, _maxNumberOfColumns, _portraitColumnCount;

		UIView _resizeThumb;
		PointF _originPoint;

		float _columnSize, _resizeThumbSize,_resizeThumbOffset ;

		List<PhotoInfo> _photos;

		SizeF _landscapeSize, _portraitSize;

		public PhotoViewerViewController () : base ("PhotoViewerViewController", null)
		{

		}



		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();

			this.View.BackgroundColor = UIColor.FromWhiteAlpha (.1f, 1);

			float thumbGridHeight = 150; 

			_columnSize = thumbGridHeight;

			_numberOfColumns = 2;
			_resizeThumbSize = 50;
			_maxNumberOfColumns = 3;
			_portraitColumnCount = 6;
			_resizeThumbOffset = 20;
		

			_landscapeSize = new SizeF(UIScreen.MainScreen.Bounds.Height, UIScreen.MainScreen.Bounds.Width);
			_portraitSize = new SizeF(UIScreen.MainScreen.Bounds.Width, UIScreen.MainScreen.Bounds.Height);

			_photoGridView = new IGGridView (new RectangleF (0, 0, this.View.Frame.Size.Width, this.View.Frame.Size.Height - thumbGridHeight), IGGridViewStyle.IGGridViewStyleSingleCellPaging);
			_photoGridView.Delegate = new PhotoGridDelegate(this);
			_photoGridView.SelectionType = IGGridViewSelectionType.IGGridViewSelectionTypeNone;
			_photoGridView.AlwaysBounceVertical = false;
			_photoGridView.AllowHorizontalBounce = true;
			this.View.AddSubview (_photoGridView);

			_resizeThumb = new UIView ();
			_resizeThumb.BackgroundColor = UIColor.Clear;
			UIPanGestureRecognizer pan = new UIPanGestureRecognizer (this, new MonoTouch.ObjCRuntime.Selector ("pan:"));
			_resizeThumb.AddGestureRecognizer (pan);
			this.View.AddSubview (_resizeThumb);

			_thumbsGridView = new IGGridView (new RectangleF (0, _photoGridView.Frame.Height, this.View.Frame.Size.Width, thumbGridHeight), IGGridViewStyle.IGGridViewStyleDefault);
			_thumbsGridView.RowSeparatorHeight = 0; 
			_thumbsGridView.Delegate = new ThumbGridDelegate(this);
			_thumbsGridView.SelectionType = IGGridViewSelectionType.IGGridViewSelectionTypeCell;
			_thumbsGridView.HeaderHeight = 0; 
			_thumbsGridView.ColumnWidth = IGColumnWidth.CreateNumericColumnWidth (thumbGridHeight);
			_thumbsGridView.RowHeight = thumbGridHeight;
			_thumbsGridView.AllowHorizontalBounce = true;
			_thumbsGridView.AlwaysBounceHorizontal = true;
			_thumbsGridView.AlwaysBounceVertical = false;
			this.View.AddSubview (_thumbsGridView);

			_photos = new List<PhotoInfo> ();
			for(int i = 1; i <= 26; i++)
			{
				PhotoInfo photoInfo = new PhotoInfo ();
				photoInfo.ImagePath = string.Format ("Photos/photo{0}.jpeg", i);
				photoInfo.ThumbPath = string.Format ("Thumbnails/thumb{0}.jpeg", i);
				_photos.Add (photoInfo);
			}
		

			IGGridViewImageColumnDefinition col = new IGGridViewImageColumnDefinition ("image", IGGridViewImageColumnDefinitionPropertyType.IGGridViewImageColumnDefinitionPropertyTypeImage);
			col.EnableZooming = true;

			_photoDS = new IGGridViewSingleRowSingleFieldDataSourceHelper (col);
			_photoDS.Data = _photos.ToArray();

			_photoGridView.DataSource = _photoDS;

			IGGridViewImageColumnDefinition thumbCol = new IGGridViewImageColumnDefinition ("thumb", IGGridViewImageColumnDefinitionPropertyType.IGGridViewImageColumnDefinitionPropertyTypeImage);
	
			_singleRowThumbDS = new IGGridViewSingleRowSingleFieldDataSourceHelper (thumbCol);
			_singleRowThumbDS.Data = _photos.ToArray();

			_thumbsDS = new IGGridViewSingleFieldMultiColumnDataSourceHelper (thumbCol);
			_thumbsDS.NumberOfColumns = 1;
			_thumbsDS.Data = _photos.ToArray();

			_thumbsGridView.DataSource = _thumbsDS;
		}

		public override void ViewWillAppear (bool animated)
		{
			base.ViewWillAppear(animated);

			this.UpdateLayoutForOrientation (this.InterfaceOrientation, 0);

		}

		public override void WillRotate (UIInterfaceOrientation toInterfaceOrientation, double duration)
		{
			base.WillRotate (toInterfaceOrientation, duration);

			this.UpdateLayoutForOrientation (toInterfaceOrientation,  duration);
		}

		public void ThumbSelected(IGCellPath path)
		{
			IGGridViewDataSourceHelper ds = (IGGridViewDataSourceHelper)_thumbsGridView.DataSource;

			path = ds.NormalizePath (path);

			path = _photoDS.DenormalizePath (path);

			_photoGridView.ScrollToCell (path, IGGridViewScrollPosition.IGGridViewScrollPositionTopLeft, false);


		}

		public void PageChanged(IGCellPath path)
		{
			if(_thumbsGridView.DataSource == _thumbsDS)
			{
				path = _photoDS.NormalizePath (path);
				path = _thumbsDS.DenormalizePath (path);
			}

			_thumbsGridView.SelectCell (path, false, IGGridViewScrollPosition.IGGridViewScrollPositionNone);
			_thumbsGridView.ScrollToCell(path, IGGridViewScrollPosition.IGGridViewScrollPositionNone, true);
			this.ThumbSelected(path);

		}

		public void UpdateLayoutForOrientation(UIInterfaceOrientation interfaceOrientation,  double duration)
		{
			float size = _columnSize* _numberOfColumns;

			if(interfaceOrientation == UIInterfaceOrientation.LandscapeLeft || interfaceOrientation == UIInterfaceOrientation.LandscapeRight)
			{
				UIView.Animate (duration, new NSAction (() =>
				{
					_thumbsGridView.Frame  = new RectangleF(0, 0, size, _landscapeSize.Height);
					_resizeThumb.Frame = new RectangleF(size,0, _resizeThumbSize, _landscapeSize.Height);
					_photoGridView.Frame = new RectangleF(size + _resizeThumbSize, 0, _landscapeSize.Width - (size + _resizeThumbSize), _landscapeSize.Height);
				 }));


				List<UIView> subViews = new List<UIView>(_resizeThumb.Subviews);
				foreach(UIView view in subViews)
					view.RemoveFromSuperview();
					
				float left = _resizeThumbOffset;
				for(int i = 0; i < 3; i++)
				{
					UIView block = new UIView ();
					block.BackgroundColor = UIColor.Gray;
					_resizeThumb.AddSubview (block);
				
					block.Frame = new RectangleF(left, _landscapeSize.Height/2 - (_resizeThumbOffset/2), 2, _resizeThumbOffset);	

					left += 5;
				}

				this.UpdateDataForOrientation(true);

			}
			else
			{
				UIView.Animate (duration, new NSAction (() =>
				                                        {
					_thumbsGridView.Frame  = new RectangleF(0, _portraitSize.Height - size, _portraitSize.Width, size);
					_resizeThumb.Frame = new RectangleF(0, _portraitSize.Height - (size + _resizeThumbSize), _portraitSize.Width, _resizeThumbSize);
					_photoGridView.Frame = new RectangleF(0, 0, _portraitSize.Width, _portraitSize.Height - (size + _resizeThumbSize));
				}));


				List<UIView> subViews = new List<UIView>(_resizeThumb.Subviews);
				foreach(UIView view in subViews)
					view.RemoveFromSuperview();

				float top = _resizeThumbOffset; 
				for(int i = 0; i < 3; i++)
				{
					UIView block = new UIView ();
					block.BackgroundColor = UIColor.Gray;
					_resizeThumb.AddSubview (block);
					block.Frame = new RectangleF(_portraitSize.Width/2 - (_resizeThumbOffset/2), top, _resizeThumbOffset, 2);
					top += 5;
				}


				this.UpdateDataForOrientation(false);

			}
		}

		[Export("pan:")]
		public void Pan(UIPanGestureRecognizer gesture)
		{
			if(gesture.State == UIGestureRecognizerState.Began)
				_originPoint = _resizeThumb.Frame.Location;

			PointF currentPoint = gesture.TranslationInView (_resizeThumb.Superview);

			if(this.InterfaceOrientation == UIInterfaceOrientation.LandscapeLeft || this.InterfaceOrientation == UIInterfaceOrientation.LandscapeRight)
			{
				float newX = _originPoint.X + currentPoint.X;
				if(newX < _columnSize)
					newX = _columnSize; 
				else if(newX > (_maxNumberOfColumns * _columnSize))
					newX = (_maxNumberOfColumns * _columnSize); 

				_numberOfColumns = ((int)(newX/_columnSize));

				float duration = 0;
				if(gesture.State == UIGestureRecognizerState.Ended)
				{
					newX = _numberOfColumns * _columnSize;
					duration = .3f;
				}

				UIView.Animate (duration, new NSAction (() =>
				                                        {
					_resizeThumb.Frame = new RectangleF(newX,_resizeThumb.Frame.Y, _resizeThumb.Frame.Width, _resizeThumb.Frame.Height);
					_photoGridView.Frame = new RectangleF(newX + _resizeThumb.Frame.Width, 0, _landscapeSize.Width - newX - _resizeThumb.Frame.Width, _photoGridView.Frame.Height);
					_thumbsGridView.Frame = new RectangleF(0,0, _numberOfColumns * _columnSize, _thumbsGridView.Frame.Height);
				}));



				if(_numberOfColumns != _thumbsDS.NumberOfColumns)
				{
					this.UpdateDataForOrientation (true);
				}
			}
			else
			{
				float newY = (this.View.Frame.Height - (_originPoint.Y + currentPoint.Y)) - _resizeThumbSize;
				if(newY < _columnSize)
					newY = _columnSize; 
				else if(newY > (_maxNumberOfColumns * _columnSize))
					newY = (_maxNumberOfColumns * _columnSize); 

				_numberOfColumns = ((int)(newY/_columnSize));

				float duration = 0;
				if(gesture.State == UIGestureRecognizerState.Ended)
				{
					newY = _numberOfColumns * _columnSize;
					duration = .3f;
				}

				int previousColumns = (int)((this.View.Frame.Height - (_resizeThumb.Frame.Location.Y + _resizeThumbSize))/_columnSize);

				UIView.Animate (duration, new NSAction (() =>
				                                        {
					_resizeThumb.Frame = new RectangleF(0,  this.View.Frame.Height - (newY + _resizeThumbSize), _resizeThumb.Frame.Width, _resizeThumbSize);
					_photoGridView.Frame = new RectangleF(0, 0, _photoGridView.Frame.Width, _portraitSize.Height - (newY + _resizeThumbSize));
					_thumbsGridView.Frame  = new RectangleF(0, this.View.Frame.Height - newY, this.View.Frame.Width, newY);
				}));

				if(_numberOfColumns != previousColumns)
					this.UpdateDataForOrientation (false);

			}
		}

		public void UpdateDataForOrientation(bool isLandscape)
		{
			IGCellPath path = _thumbsGridView.SelectedCellPath();
			if (path == null)
				path = new IGCellPath (0, 0, 0);

			IGGridViewDataSourceHelper ds = (IGGridViewDataSourceHelper)_thumbsGridView.DataSource;
			path = ds.NormalizePath (path);

			if(isLandscape)
			{
				_thumbsGridView.DataSource = _thumbsDS;
				_thumbsGridView.ColumnWidth = IGColumnWidth.CreatePercentColumnWidth (1);
				_thumbsDS.NumberOfColumns = _numberOfColumns;
				_thumbsDS.InvalidateData ();


				_thumbsGridView.AllowHorizontalBounce = false;
				_thumbsGridView.AlwaysBounceHorizontal = false;
				_thumbsGridView.AlwaysBounceVertical = true;
			}
			else
			{
				if(_numberOfColumns == 1)
				{
					_thumbsGridView.DataSource = _singleRowThumbDS;
					_thumbsGridView.ColumnWidth =  IGColumnWidth.CreateNumericColumnWidth (_portraitSize.Width/_portraitColumnCount);
					_thumbsGridView.AllowHorizontalBounce = true;
					_thumbsGridView.AlwaysBounceHorizontal = true;
					_thumbsGridView.AlwaysBounceVertical = false;
				}
				else
				{
					_thumbsGridView.DataSource = _thumbsDS;
					_thumbsGridView.ColumnWidth = IGColumnWidth.CreatePercentColumnWidth (1);
					_thumbsDS.NumberOfColumns = _portraitColumnCount;
					_thumbsDS.InvalidateData ();

					_thumbsGridView.AllowHorizontalBounce = false;
					_thumbsGridView.AlwaysBounceHorizontal = false;
					_thumbsGridView.AlwaysBounceVertical = true;
				}
			}

			_thumbsGridView.UpdateData ();


			IGGridViewDataSourceHelper currentDS = (IGGridViewDataSourceHelper)_thumbsGridView.DataSource;
			path = currentDS.DenormalizePath (path);

			_thumbsGridView.SelectCell (path, false, IGGridViewScrollPosition.IGGridViewScrollPositionNone);
			_thumbsGridView.ScrollToCell (path, IGGridViewScrollPosition.IGGridViewScrollPositionNone, false);
		}

	}



}

